import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as st
import pystan as pyst
import seaborn as sns
import patsy
sns.set(context='talk', style='ticks', font_scale=1.2, rc={'figure.figsize': (6.5, 5.5), 'xtick.direction': 'in', 'ytick.direction': 'in'})

# read in data
cumulativeData = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Data/cumulativeData.csv')
cumulativeData = cumulativeData[cumulativeData['Notes'].isnull()]
# remove gap 5
cumulativeData = cumulativeData[cumulativeData['gap']!=5]
# calculate summer mortality
cumulativeData['summer_mort']=cumulativeData[['dead_2012_summer', 'dead_2013_summer', 'dead_2014_summer']].sum(axis=1)
# calculate mortality for each gap year
cumulativeData['Year1_mort'] = cumulativeData.apply(lambda x: x['dead_2013_summer'] if x['gap']==8 else x['dead_2012_summer'], axis=1)
cumulativeData['Year2_mort'] = cumulativeData.apply(lambda x: x['dead_2014_summer'] if x['gap']==8 else x['dead_2013_summer'], axis=1)
cumulativeData['Year3_mort'] = cumulativeData.apply(lambda x: np.nan if x['gap']==8 else x['dead_2014_summer'], axis=1)
# isolate the germling data
germlings2012 = cumulativeData[cumulativeData['initial_2012'] < 15]
germlings2013 = cumulativeData[(cumulativeData['initial_2013'] < 15) & (cumulativeData['initial_2012'].isnull())]
germlings2014 = cumulativeData[(cumulativeData['initial_2014'] < 15) & (cumulativeData['initial_2013'].isnull()) & (cumulativeData['initial_2012'].isnull())]
germlings = pd.concat([germlings2012, germlings2013, germlings2014], axis=0)

# extract the relevant data
paredData = germlings[['gap', 'plot', 'treatment', 'tagID', 'scientific', 'Year1_mort', 'Year2_mort', 'Year3_mort']]
# put into long format
paredLong = pd.melt(paredData, id_vars=['gap', 'plot', 'treatment', 'tagID', 'scientific'], var_name='GapYear', value_name='Dead')
# calculate proportion dead in each plot
plotData = paredLong.groupby(['gap', 'plot', 'treatment', 'scientific', 'GapYear'], as_index=False)['Dead'].agg({'Ndead': lambda x: x.sum(), 'Ntotal': lambda x: x.count(), 'Dead': lambda x: x.mean()})
plotData.dropna(inplace=True)
# isolate common species
common = ['Acer rubrum', 'Fagus grandifolia',  'Fraxinus', 'Liriodendron tulipifera', 'Liquidambar styraciflua']
plotData = plotData[plotData['scientific'].isin(common)]
# rename the gap year levels
renameDict = {'Year1_mort': 'Year 1', 'Year2_mort': 'Year 2', 'Year3_mort': 'Year 3'}
plotData['GapYear'].replace(renameDict, inplace=True)
plotData = plotData[plotData['Ntotal']>0]

mortalityModel = """
data{
	int<lower=0> N; 		// number of observations
	int<lower=0> J;		// number of species
	row_vector[6] X[N];		// design matrix
	int total[N];			// response variable
	int dead[N];
	int species[N];			// species identifier
	matrix[6,6] R;			// priors for covariance matrix
}
parameters{
	vector[6] B[J];					// a unique vector matrix for each species
	vector[6] B_mu;				// overall coefficients
	cov_matrix[6] Tau;				// covariance matrix
}
transformed parameters{
	vector[N] yhat;
	vector[N] theta;

	for(n in 1:N){
		yhat[n] <- X[n]*B[species[n]];		// species-level regression
		theta[n] <- inv_logit(yhat[n]);
	}
}
model{
	dead ~ binomial(total, theta);

	for(j in 1:J){
		B[j] ~ multi_normal(B_mu, Tau);	// species-level coefficients vary normally around overall coefs
	}

	// PRIORS
	B_mu ~ normal(0, 1);	// Priors on overall effects
	Tau ~ inv_wishart(7, R);	// Priors on covariance of effects
}
generated quantities{	
}
"""
mortMod_comp = pyst.StanModel(model_code=mortalityModel)

# data setup
spMap = {'Acer rubrum': 1, 'Fagus grandifolia': 2,  'Fraxinus': 3, 'Liriodendron tulipifera': 4, 'Liquidambar styraciflua': 5}
X = patsy.dmatrix('~C(treatment)*C(GapYear)', data=plotData)
X_names = X.design_info.column_names

mortData = {
	'N': len(plotData),
	'J': len(plotData['scientific'].unique()),
	'X': np.array(X),
	'total': plotData['Ntotal'].astype(int),
	'dead': plotData['Ndead'].astype(int),
	'species': plotData['scientific'].map(spMap),
	'R': np.eye(6)
}

mortFitted = mortMod_comp.sampling(data=mortData, iter=10000, chains=4, thin=10)

#### SPECIES-LEVEL TRENDS ####
B = mortFitted.extract('B')['B']

Maple_coefs = pd.DataFrame(B[:,0,:])
Maple_coefs.columns = X_names
Maple_coefs.plot(kind='kde', legend=False)
plt.show()
Maple_coefs['Species'] = 'Acer rubrum'

Beech_coefs = pd.DataFrame(B[:,1,:])
Beech_coefs.columns = X_names
Beech_coefs.plot(kind='kde', legend=False)
plt.show()
Beech_coefs['Species'] = 'Fagus grandifolia'

Ash_coefs = pd.DataFrame(B[:,2,:])
Ash_coefs.columns = X_names
Ash_coefs.plot(kind='kde', legend=False)
plt.show()
Ash_coefs['Species'] = 'Fraxinus'

Tulip_coefs = pd.DataFrame(B[:,3,:])
Tulip_coefs.columns = X_names
Tulip_coefs.plot(kind='kde', legend=False)
plt.show()
Tulip_coefs['Species'] = 'Liriodendron tulipifera'

Sweet_coefs = pd.DataFrame(B[:,4,:])
Sweet_coefs.columns = X_names
Sweet_coefs.plot(kind='kde', legend=False)
plt.show()
Sweet_coefs['Species'] = 'Liquidambar styraciflua'

species_coefs = pd.concat([Maple_coefs, Beech_coefs, Ash_coefs, Tulip_coefs, Sweet_coefs])
species_coefs.to_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/species_coefs.csv', index=False)
species_coefs = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/species_coefs.csv')

f,ax = plt.subplots(1,5, figsize=(15, 5.5))
parmNames = ['Intercept', 'Pesticide', 'Year 2', 'Year 3', 'Pesticide:Year 2', 'Pesticide:Year 3']
for i in range(5):
	currentSp = common[i]
	tempData = species_coefs[species_coefs['Species']==currentSp]
	ax[i].axvline(0, c='k', ls='--')
	ax[i].hlines(range(6)[::-1], tempData.quantile(0.025), tempData.quantile(0.975), lw=1)
	ax[i].hlines(range(6)[::-1], tempData.quantile(0.1), tempData.quantile(0.9), lw=2)
	[ax[i].text(x,y,s, ha='left', va='center') for x,y,s in zip(tempData.quantile(0.98), range(6)[::-1], np.round(np.mean(tempData<0), 3).astype('str'))]
	ax[i].plot(tempData.median(), range(6)[::-1], 'o', ms=10, mew=1, mfc='w')
	ax[i].set_ylim([-0.5, 5.5])
	ax[i].set_yticks(range(6))
	ax[i].set_title(currentSp.replace(' ', '\n'))
	ax[i].set_xlim([-5, 5])
	if i==0:
		ax[i].set_yticklabels(parmNames[::-1])
	else:
		ax[i].set_yticklabels('')

sns.despine()
plt.subplots_adjust(wspace=0.3)
plt.savefig('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/mortality_coefs.pdf')
plt.show()

# species plots
cMap = {'Control': ['white', 'Herbivores'], 'Pesticide': ['black', 'No Herbivores']}
plotSumm = plotData.groupby(['scientific', 'GapYear', 'treatment'])['Dead'].agg({'mean': lambda x: x.mean(), 'SE': lambda x: x.std()/np.sqrt(x.count())})
common2 = ['Acer rubrum', 'Fagus grandifolia',  'Fraxinus spp.', 'Liriodendron tulipifera', 'Liquidambar styraciflua']

f = plt.figure(figsize=(10, 10))
for i in range(5):
	spData = plotSumm.loc[common[i]]
	spData['location'] = [0.9, 1.1, 1.9, 2.1, 2.9, 3.1]
	ax = f.add_subplot(2, 3, i+1)
	for treat in ['Control', 'Pesticide']:
		trtData = spData.xs(treat, level=1)
		ax.errorbar(trtData['location'], np.array(trtData['mean']), yerr=trtData['SE'], fmt='o', 
			mew=1, mfc=cMap[treat][0], label=cMap[treat][1], ms=15, capsize=0, ecolor='k')
	ax.set_ylim([-0.05, 1])
	ax.set_xlim([0.5, 3.5])
	ax.set_xticks([1, 2, 3])
	ax.set_title(common2[i].replace(' ', '\n'), style='italic')
	if i==0:
		ax.legend(fontsize=12, markerscale=0.8, loc=2)
	if i in [2, 3,4]:
		ax.set_xticklabels(['Year 1', 'Year 2', 'Year 3'])
	else:
		ax.set_xticklabels('')
	if i not in [1,2,4]:
		ax.set_ylabel('Mortality Rate\n(proportion dead per year)')

plt.subplots_adjust(wspace=0.3, hspace=0.3)
sns.despine()
plt.savefig('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/species_mortality.pdf')
plt.show()

# acer rubrum
Maple_coefs = species_coefs[species_coefs['Species']=='Acer rubrum']
#pesticide effect
print np.round(Maple_coefs.iloc[:,1].quantile([0.025, 0.5, 0.975]), 3)
# diff from year 1 to year 2
print Maple_coefs.iloc[:,2].quantile([0.025, 0.5, 0.975])
# diff from year 1 to year 3
print Maple_coefs.iloc[:,3].quantile([0.025, 0.5, 0.975])
# diff from year 2 to year 3 
print 1-np.mean((Maple_coefs.iloc[:,2] - Maple_coefs.iloc[:,3]) < 0)
print (Maple_coefs.iloc[:,2] - Maple_coefs.iloc[:,3]).quantile([0.025, 0.5, 0.975])

# liriodendron tulipifera
Tulip_coefs = species_coefs[species_coefs['Species']=='Liriodendron tulipifera']
#pesticide effect
print np.round(Tulip_coefs.iloc[:,1].quantile([0.025, 0.5, 0.975]), 3)
# diff from year 1 to year 2
print Tulip_coefs.iloc[:,2].quantile([0.025, 0.5, 0.975])
# diff from year 1 to year 3
print Tulip_coefs.iloc[:,3].quantile([0.025, 0.5, 0.975])
# diff from year 2 to year 3 
print 1-np.mean((Tulip_coefs.iloc[:,2] - Tulip_coefs.iloc[:,3]) < 0)
print (Tulip_coefs.iloc[:,2] - Tulip_coefs.iloc[:,3]).quantile([0.025, 0.5, 0.975])

# fraxinus
Fraxinus_coefs = species_coefs[species_coefs['Species']=='Fraxinus']
#pesticide effect
print np.round(Fraxinus_coefs.iloc[:,1].quantile([0.025, 0.5, 0.975]), 3)
# diff from year 1 to year 2
print Fraxinus_coefs.iloc[:,2].quantile([0.025, 0.5, 0.975])
# diff from year 1 to year 3
print Fraxinus_coefs.iloc[:,3].quantile([0.025, 0.5, 0.975])
# diff from year 2 to year 3 
print 1-np.mean((Fraxinus_coefs.iloc[:,2] - Fraxinus_coefs.iloc[:,3]) < 0)
print (Fraxinus_coefs.iloc[:,2] - Fraxinus_coefs.iloc[:,3]).quantile([0.025, 0.5, 0.975])

#### OVERALL COEFS
B_mu = pd.DataFrame(mortFitted.extract('B_mu')['B_mu'])
B_mu.columns = X_names
B_mu.plot(kind='kde', legend=False)
plt.show()
B_mu.to_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/overall_coefs.csv', index=False)
B_mu = pd.read_csv('/Users/Nate/Documents/FIU/Research/Gap_Dynamics/Analyses/Mortality/overall_coefs.csv')

print B_mu.iloc[:,1].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,1] < 0)

print B_mu.iloc[:,2].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,2] < 0)

print B_mu.iloc[:,3].quantile([0.025, 0.5, 0.975])
print np.mean(B_mu.iloc[:,3] < 0)